@@ -5,7 +5,12 @@ import sys
 import weakref
 
 from sentry_sdk.hub import Hub, _should_send_default_pii
-from sentry_sdk.utils import capture_internal_exceptions, event_from_exception
+from sentry_sdk.scope import Scope
+from sentry_sdk.tracing import SOURCE_FOR_STYLE
+from sentry_sdk.utils import (
+    capture_internal_exceptions,
+    event_from_exception,
+)
 from sentry_sdk._compat import reraise, iteritems
 
 from sentry_sdk.integrations import Integration, DidNotEnable
@@ -18,9 +23,9 @@ try:
 except ImportError:
     raise DidNotEnable("Pyramid not installed")
 
-from sentry_sdk._types import MYPY
+from sentry_sdk._types import TYPE_CHECKING
 
-if MYPY:
+if TYPE_CHECKING:
     from pyramid.response import Response
     from typing import Any
     from sentry_sdk.integrations.wsgi import _ScopedResponse
@@ -51,7 +56,7 @@ TRANSACTION_STYLE_VALUES = ("route_name", "route_pattern")
 class PyramidIntegration(Integration):
     identifier = "pyramid"
 
-    transaction_style = None
+    transaction_style = ""
 
     def __init__(self, transaction_style="route_name"):
         # type: (str) -> None
@@ -76,14 +81,9 @@ class PyramidIntegration(Integration):
 
             if integration is not None:
                 with hub.configure_scope() as scope:
-                    try:
-                        if integration.transaction_style == "route_name":
-                            scope.transaction = request.matched_route.name
-                        elif integration.transaction_style == "route_pattern":
-                            scope.transaction = request.matched_route.pattern
-                    except Exception:
-                        pass
-
+                    _set_transaction_name_and_source(
+                        scope, integration.transaction_style, request
+                    )
                     scope.add_event_processor(
                         _make_event_processor(weakref.ref(request), integration)
                     )
@@ -156,6 +156,21 @@ def _capture_exception(exc_info):
     hub.capture_event(event, hint=hint)
 
 
+def _set_transaction_name_and_source(scope, transaction_style, request):
+    # type: (Scope, str, Request) -> None
+    try:
+        name_for_style = {
+            "route_name": request.matched_route.name,
+            "route_pattern": request.matched_route.pattern,
+        }
+        scope.set_transaction_name(
+            name_for_style[transaction_style],
+            source=SOURCE_FOR_STYLE[transaction_style],
+        )
+    except Exception:
+        pass
+
+
 class PyramidRequestExtractor(RequestExtractor):
     def url(self):
         # type: () -> str
