@@ -1,23 +1,21 @@
-import re
 import contextlib
-import json
-import math
-
-from numbers import Real
+import os
+import re
+import sys
 
 import sentry_sdk
-
+from sentry_sdk.consts import OP, SPANDATA
 from sentry_sdk.utils import (
     capture_internal_exceptions,
     Dsn,
-    logger,
-    safe_str,
-    to_base64,
+    match_regex_list,
     to_string,
-    from_base64,
+    is_sentry_url,
+    _is_external_source,
+    _module_in_list,
 )
 from sentry_sdk._compat import PY2, iteritems
-from sentry_sdk._types import MYPY
+from sentry_sdk._types import TYPE_CHECKING
 
 if PY2:
     from collections import Mapping
@@ -26,15 +24,17 @@ else:
     from collections.abc import Mapping
     from urllib.parse import quote, unquote
 
-if MYPY:
+if TYPE_CHECKING:
     import typing
 
-    from typing import Generator
-    from typing import Optional
     from typing import Any
     from typing import Dict
+    from typing import Generator
+    from typing import Optional
     from typing import Union
 
+    from types import FrameType
+
 
 SENTRY_TRACE_REGEX = re.compile(
     "^[ \t]*"  # whitespace
@@ -55,27 +55,6 @@ base64_stripped = (
     "([a-zA-Z0-9+/]{2,3})?"
 )
 
-# comma-delimited list of entries of the form `xxx=yyy`
-tracestate_entry = "[^=]+=[^=]+"
-TRACESTATE_ENTRIES_REGEX = re.compile(
-    # one or more xxxxx=yyyy entries
-    "^({te})+"
-    # each entry except the last must be followed by a comma
-    "(,|$)".format(te=tracestate_entry)
-)
-
-# this doesn't check that the value is valid, just that there's something there
-# of the form `sentry=xxxx`
-SENTRY_TRACESTATE_ENTRY_REGEX = re.compile(
-    # either sentry is the first entry or there's stuff immediately before it,
-    # ending in a comma (this prevents matching something like `coolsentry=xxx`)
-    "(?:^|.+,)"
-    # sentry's part, not including the potential comma
-    "(sentry=[^,]*)"
-    # either there's a comma and another vendor's entry or we end
-    "(?:,.+|$)"
-)
-
 
 class EnvironHeaders(Mapping):  # type: ignore
     def __init__(
@@ -109,49 +88,23 @@ class EnvironHeaders(Mapping):  # type: ignore
 
 
 def has_tracing_enabled(options):
-    # type: (Dict[str, Any]) -> bool
+    # type: (Optional[Dict[str, Any]]) -> bool
     """
     Returns True if either traces_sample_rate or traces_sampler is
-    defined, False otherwise.
+    defined and enable_tracing is set and not false.
     """
+    if options is None:
+        return False
 
     return bool(
-        options.get("traces_sample_rate") is not None
-        or options.get("traces_sampler") is not None
+        options.get("enable_tracing") is not False
+        and (
+            options.get("traces_sample_rate") is not None
+            or options.get("traces_sampler") is not None
+        )
     )
 
 
-def is_valid_sample_rate(rate):
-    # type: (Any) -> bool
-    """
-    Checks the given sample rate to make sure it is valid type and value (a
-    boolean or a number between 0 and 1, inclusive).
-    """
-
-    # both booleans and NaN are instances of Real, so a) checking for Real
-    # checks for the possibility of a boolean also, and b) we have to check
-    # separately for NaN
-    if not isinstance(rate, Real) or math.isnan(rate):
-        logger.warning(
-            "[Tracing] Given sample rate is invalid. Sample rate must be a boolean or a number between 0 and 1. Got {rate} of type {type}.".format(
-                rate=rate, type=type(rate)
-            )
-        )
-        return False
-
-    # in case rate is a boolean, it will get cast to 1 if it's True and 0 if it's False
-    rate = float(rate)
-    if rate < 0 or rate > 1:
-        logger.warning(
-            "[Tracing] Given sample rate is invalid. Sample rate must be between 0 and 1. Got {rate}.".format(
-                rate=rate
-            )
-        )
-        return False
-
-    return True
-
-
 @contextlib.contextmanager
 def record_sql_queries(
     hub,  # type: sentry_sdk.Hub
@@ -160,8 +113,9 @@ def record_sql_queries(
     params_list,  # type:  Any
     paramstyle,  # type: Optional[str]
     executemany,  # type: bool
+    record_cursor_repr=False,  # type: bool
 ):
-    # type: (...) -> Generator[Span, None, None]
+    # type: (...) -> Generator[sentry_sdk.tracing.Span, None, None]
 
     # TODO: Bring back capturing of params by default
     if hub.client and hub.client.options["_experiments"].get(
@@ -185,23 +139,25 @@ def record_sql_queries(
         data["db.paramstyle"] = paramstyle
     if executemany:
         data["db.executemany"] = True
+    if record_cursor_repr and cursor is not None:
+        data["db.cursor"] = cursor
 
     with capture_internal_exceptions():
         hub.add_breadcrumb(message=query, category="query", data=data)
 
-    with hub.start_span(op="db", description=query) as span:
+    with hub.start_span(op=OP.DB, description=query) as span:
         for k, v in data.items():
             span.set_data(k, v)
         yield span
 
 
 def maybe_create_breadcrumbs_from_span(hub, span):
-    # type: (sentry_sdk.Hub, Span) -> None
-    if span.op == "redis":
+    # type: (sentry_sdk.Hub, sentry_sdk.tracing.Span) -> None
+    if span.op == OP.DB_REDIS:
         hub.add_breadcrumb(
             message=span.description, type="redis", category="redis", data=span._tags
         )
-    elif span.op == "http":
+    elif span.op == OP.HTTP_CLIENT:
         hub.add_breadcrumb(type="http", category="httplib", data=span._data)
     elif span.op == "subprocess":
         hub.add_breadcrumb(
@@ -212,8 +168,110 @@ def maybe_create_breadcrumbs_from_span(hub, span):
         )
 
 
+def add_query_source(hub, span):
+    # type: (sentry_sdk.Hub, sentry_sdk.tracing.Span) -> None
+    """
+    Adds OTel compatible source code information to the span
+    """
+    client = hub.client
+    if client is None:
+        return
+
+    if span.timestamp is None or span.start_timestamp is None:
+        return
+
+    should_add_query_source = client.options.get("enable_db_query_source", False)
+    if not should_add_query_source:
+        return
+
+    duration = span.timestamp - span.start_timestamp
+    threshold = client.options.get("db_query_source_threshold_ms", 0)
+    slow_query = duration.microseconds > threshold * 1000
+
+    if not slow_query:
+        return
+
+    project_root = client.options["project_root"]
+    in_app_include = client.options.get("in_app_include")
+    in_app_exclude = client.options.get("in_app_exclude")
+
+    # Find the correct frame
+    frame = sys._getframe()  # type: Union[FrameType, None]
+    while frame is not None:
+        try:
+            abs_path = frame.f_code.co_filename
+            if abs_path and PY2:
+                abs_path = os.path.abspath(abs_path)
+        except Exception:
+            abs_path = ""
+
+        try:
+            namespace = frame.f_globals.get("__name__")  # type: Optional[str]
+        except Exception:
+            namespace = None
+
+        is_sentry_sdk_frame = namespace is not None and namespace.startswith(
+            "sentry_sdk."
+        )
+
+        should_be_included = not _is_external_source(abs_path)
+        if namespace is not None:
+            if in_app_exclude and _module_in_list(namespace, in_app_exclude):
+                should_be_included = False
+            if in_app_include and _module_in_list(namespace, in_app_include):
+                # in_app_include takes precedence over in_app_exclude, so doing it
+                # at the end
+                should_be_included = True
+
+        if (
+            abs_path.startswith(project_root)
+            and should_be_included
+            and not is_sentry_sdk_frame
+        ):
+            break
+
+        frame = frame.f_back
+    else:
+        frame = None
+
+    # Set the data
+    if frame is not None:
+        try:
+            lineno = frame.f_lineno
+        except Exception:
+            lineno = None
+        if lineno is not None:
+            span.set_data(SPANDATA.CODE_LINENO, frame.f_lineno)
+
+        try:
+            namespace = frame.f_globals.get("__name__")
+        except Exception:
+            namespace = None
+        if namespace is not None:
+            span.set_data(SPANDATA.CODE_NAMESPACE, namespace)
+
+        try:
+            filepath = frame.f_code.co_filename
+        except Exception:
+            filepath = None
+        if filepath is not None:
+            if project_root is not None and filepath.startswith(project_root):
+                in_app_path = filepath.replace(project_root, "").lstrip(os.sep)
+            else:
+                in_app_path = filepath
+            span.set_data(SPANDATA.CODE_FILEPATH, in_app_path)
+
+        try:
+            code_function = frame.f_code.co_name
+        except Exception:
+            code_function = None
+
+        if code_function is not None:
+            span.set_data(SPANDATA.CODE_FUNCTION, frame.f_code.co_name)
+
+
 def extract_sentrytrace_data(header):
-    # type: (Optional[str]) -> Optional[typing.Mapping[str, Union[str, bool, None]]]
+    # type: (Optional[str]) -> Optional[Dict[str, Union[str, bool, None]]]
     """
     Given a `sentry-trace` header string, return a dictionary of data.
     """
@@ -244,143 +302,6 @@ def extract_sentrytrace_data(header):
     }
 
 
-def extract_tracestate_data(header):
-    # type: (Optional[str]) -> typing.Mapping[str, Optional[str]]
-    """
-    Extracts the sentry tracestate value and any third-party data from the given
-    tracestate header, returning a dictionary of data.
-    """
-    sentry_entry = third_party_entry = None
-    before = after = ""
-
-    if header:
-        # find sentry's entry, if any
-        sentry_match = SENTRY_TRACESTATE_ENTRY_REGEX.search(header)
-
-        if sentry_match:
-            sentry_entry = sentry_match.group(1)
-
-            # remove the commas after the split so we don't end up with
-            # `xxx=yyy,,zzz=qqq` (double commas) when we put them back together
-            before, after = map(lambda s: s.strip(","), header.split(sentry_entry))
-
-            # extract sentry's value from its entry and test to make sure it's
-            # valid; if it isn't, discard the entire entry so that a new one
-            # will be created
-            sentry_value = sentry_entry.replace("sentry=", "")
-            if not re.search("^{b64}$".format(b64=base64_stripped), sentry_value):
-                sentry_entry = None
-        else:
-            after = header
-
-        # if either part is invalid or empty, remove it before gluing them together
-        third_party_entry = (
-            ",".join(filter(TRACESTATE_ENTRIES_REGEX.search, [before, after])) or None
-        )
-
-    return {
-        "sentry_tracestate": sentry_entry,
-        "third_party_tracestate": third_party_entry,
-    }
-
-
-def compute_tracestate_value(data):
-    # type: (typing.Mapping[str, str]) -> str
-    """
-    Computes a new tracestate value using the given data.
-
-    Note: Returns just the base64-encoded data, NOT the full `sentry=...`
-    tracestate entry.
-    """
-
-    tracestate_json = json.dumps(data, default=safe_str)
-
-    # Base64-encoded strings always come out with a length which is a multiple
-    # of 4. In order to achieve this, the end is padded with one or more `=`
-    # signs. Because the tracestate standard calls for using `=` signs between
-    # vendor name and value (`sentry=xxx,dogsaregreat=yyy`), to avoid confusion
-    # we strip the `=`
-    return (to_base64(tracestate_json) or "").rstrip("=")
-
-
-def compute_tracestate_entry(span):
-    # type: (Span) -> Optional[str]
-    """
-    Computes a new sentry tracestate for the span. Includes the `sentry=`.
-
-    Will return `None` if there's no client and/or no DSN.
-    """
-    data = {}
-
-    hub = span.hub or sentry_sdk.Hub.current
-
-    client = hub.client
-    scope = hub.scope
-
-    if client and client.options.get("dsn"):
-        options = client.options
-        user = scope._user
-
-        data = {
-            "trace_id": span.trace_id,
-            "environment": options["environment"],
-            "release": options.get("release"),
-            "public_key": Dsn(options["dsn"]).public_key,
-        }
-
-        if user and (user.get("id") or user.get("segment")):
-            user_data = {}
-
-            if user.get("id"):
-                user_data["id"] = user["id"]
-
-            if user.get("segment"):
-                user_data["segment"] = user["segment"]
-
-            data["user"] = user_data
-
-        if span.containing_transaction:
-            data["transaction"] = span.containing_transaction.name
-
-        return "sentry=" + compute_tracestate_value(data)
-
-    return None
-
-
-def reinflate_tracestate(encoded_tracestate):
-    # type: (str) -> typing.Optional[Mapping[str, str]]
-    """
-    Given a sentry tracestate value in its encoded form, translate it back into
-    a dictionary of data.
-    """
-    inflated_tracestate = None
-
-    if encoded_tracestate:
-        # Base64-encoded strings always come out with a length which is a
-        # multiple of 4. In order to achieve this, the end is padded with one or
-        # more `=` signs. Because the tracestate standard calls for using `=`
-        # signs between vendor name and value (`sentry=xxx,dogsaregreat=yyy`),
-        # to avoid confusion we strip the `=` when the data is initially
-        # encoded. Python's decoding function requires they be put back.
-        # Fortunately, it doesn't complain if there are too many, so we just
-        # attach two `=` on spec (there will never be more than 2, see
-        # https://en.wikipedia.org/wiki/Base64#Decoding_Base64_without_padding).
-        tracestate_json = from_base64(encoded_tracestate + "==")
-
-        try:
-            assert tracestate_json is not None
-            inflated_tracestate = json.loads(tracestate_json)
-        except Exception as err:
-            logger.warning(
-                (
-                    "Unable to attach tracestate data to envelope header: {err}"
-                    + "\nTracestate value is {encoded_tracestate}"
-                ).format(err=err, encoded_tracestate=encoded_tracestate),
-            )
-
-    return inflated_tracestate
-
-
 def _format_sql(cursor, sql):
     # type: (Any, str) -> Optional[str]
 
@@ -401,40 +322,16 @@ def _format_sql(cursor, sql):
     return real_sql or to_string(sql)
 
 
-def has_tracestate_enabled(span=None):
-    # type: (Optional[Span]) -> bool
-
-    client = ((span and span.hub) or sentry_sdk.Hub.current).client
-    options = client and client.options
-
-    return bool(options and options["_experiments"].get("propagate_tracestate"))
-
-
-def has_custom_measurements_enabled():
-    # type: () -> bool
-    client = sentry_sdk.Hub.current.client
-    options = client and client.options
-    return bool(options and options["_experiments"].get("custom_measurements"))
-
-
 class Baggage(object):
+    """
+    The W3C Baggage header information (see https://www.w3.org/TR/baggage/).
+    """
+
     __slots__ = ("sentry_items", "third_party_items", "mutable")
 
     SENTRY_PREFIX = "sentry-"
     SENTRY_PREFIX_REGEX = re.compile("^sentry-")
 
-    # DynamicSamplingContext
-    DSC_KEYS = [
-        "trace_id",
-        "public_key",
-        "sample_rate",
-        "release",
-        "environment",
-        "transaction",
-        "user_id",
-        "user_segment",
-    ]
-
     def __init__(
         self,
         sentry_items,  # type: Dict[str, str]
@@ -472,9 +369,46 @@ class Baggage(object):
 
         return Baggage(sentry_items, third_party_items, mutable)
 
+    @classmethod
+    def from_options(cls, scope):
+        # type: (sentry_sdk.scope.Scope) -> Optional[Baggage]
+
+        sentry_items = {}  # type: Dict[str, str]
+        third_party_items = ""
+        mutable = False
+
+        client = sentry_sdk.Hub.current.client
+
+        if client is None or scope._propagation_context is None:
+            return Baggage(sentry_items)
+
+        options = client.options
+        propagation_context = scope._propagation_context
+
+        if propagation_context is not None and "trace_id" in propagation_context:
+            sentry_items["trace_id"] = propagation_context["trace_id"]
+
+        if options.get("environment"):
+            sentry_items["environment"] = options["environment"]
+
+        if options.get("release"):
+            sentry_items["release"] = options["release"]
+
+        if options.get("dsn"):
+            sentry_items["public_key"] = Dsn(options["dsn"]).public_key
+
+        if options.get("traces_sample_rate"):
+            sentry_items["sample_rate"] = options["traces_sample_rate"]
+
+        user = (scope and scope._user) or {}
+        if user.get("segment"):
+            sentry_items["user_segment"] = user["segment"]
+
+        return Baggage(sentry_items, third_party_items, mutable)
+
     @classmethod
     def populate_from_transaction(cls, transaction):
-        # type: (Transaction) -> Baggage
+        # type: (sentry_sdk.tracing.Transaction) -> Baggage
         """
         Populate fresh baggage entry with sentry_items and make it immutable
         if this is the head SDK which originates traces.
@@ -512,6 +446,9 @@ class Baggage(object):
         if transaction.sample_rate is not None:
             sentry_items["sample_rate"] = str(transaction.sample_rate)
 
+        if transaction.sampled is not None:
+            sentry_items["sampled"] = "true" if transaction.sampled else "false"
+
         # there's an existing baggage but it was mutable,
         # which is why we are creating this new baggage.
         # However, if by chance the user put some sentry items in there, give them precedence.
@@ -528,10 +465,8 @@ class Baggage(object):
         # type: () -> Dict[str, str]
         header = {}
 
-        for key in Baggage.DSC_KEYS:
-            item = self.sentry_items.get(key)
-            if item:
-                header[key] = item
+        for key, item in iteritems(self.sentry_items):
+            header[key] = item
 
         return header
 
@@ -550,8 +485,35 @@ class Baggage(object):
         return ",".join(items)
 
 
+def should_propagate_trace(hub, url):
+    # type: (sentry_sdk.Hub, str) -> bool
+    """
+    Returns True if url matches trace_propagation_targets configured in the given hub. Otherwise, returns False.
+    """
+    client = hub.client  # type: Any
+    trace_propagation_targets = client.options["trace_propagation_targets"]
+
+    if is_sentry_url(hub, url):
+        return False
+
+    return match_regex_list(url, trace_propagation_targets, substring_matching=True)
+
+
+def normalize_incoming_data(incoming_data):
+    # type: (Dict[str, Any]) -> Dict[str, Any]
+    """
+    Normalizes incoming data so the keys are all lowercase with dashes instead of underscores and stripped from known prefixes.
+    """
+    data = {}
+    for key, value in incoming_data.items():
+        if key.startswith("HTTP_"):
+            key = key[5:]
+
+        key = key.replace("_", "-").lower()
+        data[key] = value
+
+    return data
+
+
 # Circular imports
 from sentry_sdk.tracing import LOW_QUALITY_TRANSACTION_SOURCES
-
-if MYPY:
-    from sentry_sdk.tracing import Span, Transaction
