@@ -2,12 +2,10 @@ import json
 import pytest
 import logging
 
-
-pytest.importorskip("bottle")
-
 from io import BytesIO
 from bottle import Bottle, debug as set_debug, abort, redirect
 from sentry_sdk import capture_message
+from sentry_sdk.serializer import MAX_DATABAG_BREADTH
 
 from sentry_sdk.integrations.logging import LoggingIntegration
 from werkzeug.test import Client
@@ -210,7 +208,7 @@ def test_too_large_raw_request(
     sentry_init, input_char, capture_events, app, get_client
 ):
     sentry_init(
-        integrations=[bottle_sentry.BottleIntegration()], request_bodies="small"
+        integrations=[bottle_sentry.BottleIntegration()], max_request_body_size="small"
     )
 
     data = input_char * 2000
@@ -240,7 +238,7 @@ def test_too_large_raw_request(
 
 def test_files_and_form(sentry_init, capture_events, app, get_client):
     sentry_init(
-        integrations=[bottle_sentry.BottleIntegration()], request_bodies="always"
+        integrations=[bottle_sentry.BottleIntegration()], max_request_body_size="always"
     )
 
     data = {"foo": "a" * 2000, "file": (BytesIO(b"hello"), "hello.txt")}
@@ -275,6 +273,37 @@ def test_files_and_form(sentry_init, capture_events, app, get_client):
     assert not event["request"]["data"]["file"]
 
 
+def test_json_not_truncated_if_max_request_body_size_is_always(
+    sentry_init, capture_events, app, get_client
+):
+    sentry_init(
+        integrations=[bottle_sentry.BottleIntegration()], max_request_body_size="always"
+    )
+
+    data = {
+        "key{}".format(i): "value{}".format(i) for i in range(MAX_DATABAG_BREADTH + 10)
+    }
+
+    @app.route("/", method="POST")
+    def index():
+        import bottle
+
+        assert bottle.request.json == data
+        assert bottle.request.body.read() == json.dumps(data).encode("ascii")
+        capture_message("hi")
+        return "ok"
+
+    events = capture_events()
+
+    client = get_client()
+
+    response = client.post("/", content_type="application/json", data=json.dumps(data))
+    assert response[1] == "200 OK"
+
+    (event,) = events
+    assert event["request"]["data"] == data
+
+
 @pytest.mark.parametrize(
     "integrations",
     [
@@ -354,10 +383,8 @@ def test_mount(app, capture_exceptions, capture_events, sentry_init, get_client)
     assert error is exc.value
 
     (event,) = events
-    assert event["exception"]["values"][0]["mechanism"] == {
-        "type": "bottle",
-        "handled": False,
-    }
+    assert event["exception"]["values"][0]["mechanism"]["type"] == "bottle"
+    assert event["exception"]["values"][0]["mechanism"]["handled"] is False
 
 
 def test_500(sentry_init, capture_events, app, get_client):
