@@ -1,4 +1,5 @@
 from datetime import datetime
+from datetime import timezone
 import time
 import pytest
 
@@ -331,7 +332,9 @@ def test_on_start_transaction():
             parent_span_id="abcdef1234567890",
             trace_id="1234567890abcdef1234567890abcdef",
             baggage=None,
-            start_timestamp=datetime.fromtimestamp(otel_span.start_time / 1e9),
+            start_timestamp=datetime.fromtimestamp(
+                otel_span.start_time / 1e9, timezone.utc
+            ),
             instrumenter="otel",
         )
 
@@ -376,7 +379,9 @@ def test_on_start_child():
         fake_span.start_child.assert_called_once_with(
             span_id="1234567890abcdef",
             description="Sample OTel Span",
-            start_timestamp=datetime.fromtimestamp(otel_span.start_time / 1e9),
+            start_timestamp=datetime.fromtimestamp(
+                otel_span.start_time / 1e9, timezone.utc
+            ),
             instrumenter="otel",
         )
 
