@@ -1,13 +1,9 @@
+import json
+import threading
+
 import pytest
 import pytest_asyncio
 
-quart = pytest.importorskip("quart")
-
-from quart import Quart, Response, abort, stream_with_context
-from quart.views import View
-
-from quart_auth import AuthManager, AuthUser, login_user
-
 from sentry_sdk import (
     set_tag,
     configure_scope,
@@ -18,15 +14,26 @@ from sentry_sdk import (
 from sentry_sdk.integrations.logging import LoggingIntegration
 import sentry_sdk.integrations.quart as quart_sentry
 
+from quart import Quart, Response, abort, stream_with_context
+from quart.views import View
 
-auth_manager = AuthManager()
+from quart_auth import AuthUser, login_user
+
+try:
+    from quart_auth import QuartAuth
+
+    auth_manager = QuartAuth()
+except ImportError:
+    from quart_auth import AuthManager
+
+    auth_manager = AuthManager()
 
 
 @pytest_asyncio.fixture
 async def app():
     app = Quart(__name__)
-    app.debug = True
-    app.config["TESTING"] = True
+    app.debug = False
+    app.config["TESTING"] = False
     app.secret_key = "haha"
 
     auth_manager.init_app(app)
@@ -41,6 +48,20 @@ async def app():
         capture_message("hi with id")
         return "ok with id"
 
+    @app.get("/sync/thread_ids")
+    def _thread_ids_sync():
+        return {
+            "main": str(threading.main_thread().ident),
+            "active": str(threading.current_thread().ident),
+        }
+
+    @app.get("/async/thread_ids")
+    async def _thread_ids_async():
+        return {
+            "main": str(threading.main_thread().ident),
+            "active": str(threading.current_thread().ident),
+        }
+
     return app
 
 
@@ -102,22 +123,15 @@ async def test_transaction_style(
 
 
 @pytest.mark.asyncio
-@pytest.mark.parametrize("debug", (True, False))
-@pytest.mark.parametrize("testing", (True, False))
 async def test_errors(
     sentry_init,
     capture_exceptions,
     capture_events,
     app,
-    debug,
-    testing,
     integration_enabled_params,
 ):
     sentry_init(debug=True, **integration_enabled_params)
 
-    app.debug = debug
-    app.testing = testing
-
     @app.route("/")
     async def index():
         1 / 0
@@ -302,9 +316,6 @@ def test_cli_commands_raise(app):
 async def test_500(sentry_init, capture_events, app):
     sentry_init(integrations=[quart_sentry.QuartIntegration()])
 
-    app.debug = False
-    app.testing = False
-
     @app.route("/")
     async def index():
         1 / 0
@@ -328,9 +339,6 @@ async def test_500(sentry_init, capture_events, app):
 async def test_error_in_errorhandler(sentry_init, capture_events, app):
     sentry_init(integrations=[quart_sentry.QuartIntegration()])
 
-    app.debug = False
-    app.testing = False
-
     @app.route("/")
     async def index():
         raise ValueError()
@@ -523,3 +531,30 @@ async def test_class_based_views(sentry_init, app, capture_events):
 
     assert event["message"] == "hi"
     assert event["transaction"] == "hello_class"
+
+
+@pytest.mark.parametrize("endpoint", ["/sync/thread_ids", "/async/thread_ids"])
+async def test_active_thread_id(sentry_init, capture_envelopes, endpoint, app):
+    sentry_init(
+        traces_sample_rate=1.0,
+        _experiments={"profiles_sample_rate": 1.0},
+    )
+
+    envelopes = capture_envelopes()
+
+    async with app.test_client() as client:
+        response = await client.get(endpoint)
+        assert response.status_code == 200
+
+    data = json.loads(response.content)
+
+    envelopes = [envelope for envelope in envelopes]
+    assert len(envelopes) == 1
+
+    profiles = [item for item in envelopes[0].items if item.type == "profile"]
+    assert len(profiles) == 1
+
+    for profile in profiles:
+        transactions = profile.payload.json["transactions"]
+        assert len(transactions) == 1
+        assert str(data["active"]) == transactions[0]["active_thread_id"]
