@@ -1,6 +1,5 @@
-import platform
-import sys
 import random
+
 import pytest
 
 try:
@@ -12,39 +11,48 @@ except ImportError:
 
 try:
     # py2
-    from httplib import HTTPSConnection
+    from httplib import HTTPConnection, HTTPSConnection
 except ImportError:
     # py3
-    from http.client import HTTPSConnection
+    from http.client import HTTPConnection, HTTPSConnection
 
 try:
     from unittest import mock  # python 3.3 and above
 except ImportError:
     import mock  # python < 3.3
 
+
 from sentry_sdk import capture_message, start_transaction
+from sentry_sdk.consts import MATCH_ALL, SPANDATA
 from sentry_sdk.tracing import Transaction
 from sentry_sdk.integrations.stdlib import StdlibIntegration
 
+from tests.conftest import create_mock_http_server
+
+PORT = create_mock_http_server()
+
 
 def test_crumb_capture(sentry_init, capture_events):
     sentry_init(integrations=[StdlibIntegration()])
     events = capture_events()
 
-    url = "https://httpbin.org/status/200"
-    response = urlopen(url)
-    assert response.getcode() == 200
+    url = "http://localhost:{}/some/random/url".format(PORT)
+    urlopen(url)
+
     capture_message("Testing!")
 
     (event,) = events
     (crumb,) = event["breadcrumbs"]["values"]
+
     assert crumb["type"] == "http"
     assert crumb["category"] == "httplib"
     assert crumb["data"] == {
         "url": url,
-        "method": "GET",
-        "status_code": 200,
+        SPANDATA.HTTP_METHOD: "GET",
+        SPANDATA.HTTP_STATUS_CODE: 200,
         "reason": "OK",
+        SPANDATA.HTTP_FRAGMENT: "",
+        SPANDATA.HTTP_QUERY: "",
     }
 
 
@@ -56,9 +64,9 @@ def test_crumb_capture_hint(sentry_init, capture_events):
     sentry_init(integrations=[StdlibIntegration()], before_breadcrumb=before_breadcrumb)
     events = capture_events()
 
-    url = "https://httpbin.org/status/200"
-    response = urlopen(url)
-    assert response.getcode() == 200
+    url = "http://localhost:{}/some/random/url".format(PORT)
+    urlopen(url)
+
     capture_message("Testing!")
 
     (event,) = events
@@ -67,14 +75,23 @@ def test_crumb_capture_hint(sentry_init, capture_events):
     assert crumb["category"] == "httplib"
     assert crumb["data"] == {
         "url": url,
-        "method": "GET",
-        "status_code": 200,
+        SPANDATA.HTTP_METHOD: "GET",
+        SPANDATA.HTTP_STATUS_CODE: 200,
         "reason": "OK",
         "extra": "foo",
+        SPANDATA.HTTP_FRAGMENT: "",
+        SPANDATA.HTTP_QUERY: "",
     }
 
-    if platform.python_implementation() != "PyPy":
-        assert sys.getrefcount(response) == 2
+
+def test_empty_realurl(sentry_init):
+    """
+    Ensure that after using sentry_sdk.init you can putrequest a
+    None url.
+    """
+
+    sentry_init(dsn="")
+    HTTPConnection("example.com", port=443).putrequest("POST", None)
 
 
 def test_httplib_misuse(sentry_init, capture_events, request):
@@ -90,19 +107,19 @@ def test_httplib_misuse(sentry_init, capture_events, request):
     sentry_init()
     events = capture_events()
 
-    conn = HTTPSConnection("httpbin.org", 443)
+    conn = HTTPConnection("localhost", PORT)
 
     # make sure we release the resource, even if the test fails
     request.addfinalizer(conn.close)
 
-    conn.request("GET", "/anything/foo")
+    conn.request("GET", "/200")
 
-    with pytest.raises(Exception):
+    with pytest.raises(Exception):  # noqa: B017
         # This raises an exception, because we didn't call `getresponse` for
         # the previous request yet.
         #
         # This call should not affect our breadcrumb.
-        conn.request("POST", "/anything/bar")
+        conn.request("POST", "/200")
 
     response = conn.getresponse()
     assert response._method == "GET"
@@ -115,10 +132,12 @@ def test_httplib_misuse(sentry_init, capture_events, request):
     assert crumb["type"] == "http"
     assert crumb["category"] == "httplib"
     assert crumb["data"] == {
-        "url": "https://httpbin.org/anything/foo",
-        "method": "GET",
-        "status_code": 200,
+        "url": "http://localhost:{}/200".format(PORT),
+        SPANDATA.HTTP_METHOD: "GET",
+        SPANDATA.HTTP_STATUS_CODE: 200,
         "reason": "OK",
+        SPANDATA.HTTP_FRAGMENT: "",
+        SPANDATA.HTTP_QUERY: "",
     }
 
 
@@ -146,7 +165,6 @@ def test_outgoing_trace_headers(sentry_init, monkeypatch):
         op="greeting.sniff",
         trace_id="12312012123120121231201212312012",
     ) as transaction:
-
         HTTPSConnection("www.squirrelchasers.com").request("GET", "/top-chasers")
 
         (request_str,) = mock_send.call_args[0]
@@ -210,6 +228,7 @@ def test_outgoing_trace_headers_head_sdk(sentry_init, monkeypatch):
         expected_outgoing_baggage_items = [
             "sentry-trace_id=%s" % transaction.trace_id,
             "sentry-sample_rate=0.5",
+            "sentry-sampled=%s" % "true" if transaction.sampled else "false",
             "sentry-release=foo",
             "sentry-environment=production",
         ]
@@ -217,3 +236,108 @@ def test_outgoing_trace_headers_head_sdk(sentry_init, monkeypatch):
         assert sorted(request_headers["baggage"].split(",")) == sorted(
             expected_outgoing_baggage_items
         )
+
+
+@pytest.mark.parametrize(
+    "trace_propagation_targets,host,path,trace_propagated",
+    [
+        [
+            [],
+            "example.com",
+            "/",
+            False,
+        ],
+        [
+            None,
+            "example.com",
+            "/",
+            False,
+        ],
+        [
+            [MATCH_ALL],
+            "example.com",
+            "/",
+            True,
+        ],
+        [
+            ["https://example.com/"],
+            "example.com",
+            "/",
+            True,
+        ],
+        [
+            ["https://example.com/"],
+            "example.com",
+            "",
+            False,
+        ],
+        [
+            ["https://example.com"],
+            "example.com",
+            "",
+            True,
+        ],
+        [
+            ["https://example.com", r"https?:\/\/[\w\-]+(\.[\w\-]+)+\.net"],
+            "example.net",
+            "",
+            False,
+        ],
+        [
+            ["https://example.com", r"https?:\/\/[\w\-]+(\.[\w\-]+)+\.net"],
+            "good.example.net",
+            "",
+            True,
+        ],
+        [
+            ["https://example.com", r"https?:\/\/[\w\-]+(\.[\w\-]+)+\.net"],
+            "good.example.net",
+            "/some/thing",
+            True,
+        ],
+    ],
+)
+def test_option_trace_propagation_targets(
+    sentry_init, monkeypatch, trace_propagation_targets, host, path, trace_propagated
+):
+    # HTTPSConnection.send is passed a string containing (among other things)
+    # the headers on the request. Mock it so we can check the headers, and also
+    # so it doesn't try to actually talk to the internet.
+    mock_send = mock.Mock()
+    monkeypatch.setattr(HTTPSConnection, "send", mock_send)
+
+    sentry_init(
+        trace_propagation_targets=trace_propagation_targets,
+        traces_sample_rate=1.0,
+    )
+
+    headers = {
+        "baggage": (
+            "sentry-trace_id=771a43a4192642f0b136d5159a501700, "
+            "sentry-public_key=49d0f7386ad645858ae85020e393bef3, sentry-sample_rate=0.01337, "
+        )
+    }
+
+    transaction = Transaction.continue_from_headers(headers)
+
+    with start_transaction(
+        transaction=transaction,
+        name="/interactions/other-dogs/new-dog",
+        op="greeting.sniff",
+        trace_id="12312012123120121231201212312012",
+    ) as transaction:
+        HTTPSConnection(host).request("GET", path)
+
+        (request_str,) = mock_send.call_args[0]
+        request_headers = {}
+        for line in request_str.decode("utf-8").split("\r\n")[1:]:
+            if line:
+                key, val = line.split(": ")
+                request_headers[key] = val
+
+        if trace_propagated:
+            assert "sentry-trace" in request_headers
+            assert "baggage" in request_headers
+        else:
+            assert "sentry-trace" not in request_headers
+            assert "baggage" not in request_headers
