@@ -23,7 +23,7 @@ def test_mixed_baggage():
     header = (
         "other-vendor-value-1=foo;bar;baz, sentry-trace_id=771a43a4192642f0b136d5159a501700, "
         "sentry-public_key=49d0f7386ad645858ae85020e393bef3, sentry-sample_rate=0.01337, "
-        "sentry-user_id=Am%C3%A9lie, other-vendor-value-2=foo;bar;"
+        "sentry-user_id=Am%C3%A9lie, sentry-foo=bar, other-vendor-value-2=foo;bar;"
     )
 
     baggage = Baggage.from_incoming_header(header)
@@ -35,6 +35,7 @@ def test_mixed_baggage():
         "trace_id": "771a43a4192642f0b136d5159a501700",
         "user_id": "Amélie",
         "sample_rate": "0.01337",
+        "foo": "bar",
     }
 
     assert (
@@ -47,13 +48,15 @@ def test_mixed_baggage():
         "trace_id": "771a43a4192642f0b136d5159a501700",
         "user_id": "Amélie",
         "sample_rate": "0.01337",
+        "foo": "bar",
     }
 
     assert sorted(baggage.serialize().split(",")) == sorted(
         (
             "sentry-trace_id=771a43a4192642f0b136d5159a501700,"
             "sentry-public_key=49d0f7386ad645858ae85020e393bef3,"
-            "sentry-sample_rate=0.01337,sentry-user_id=Am%C3%A9lie"
+            "sentry-sample_rate=0.01337,sentry-user_id=Am%C3%A9lie,"
+            "sentry-foo=bar"
         ).split(",")
     )
 
@@ -61,7 +64,7 @@ def test_mixed_baggage():
         (
             "sentry-trace_id=771a43a4192642f0b136d5159a501700,"
             "sentry-public_key=49d0f7386ad645858ae85020e393bef3,"
-            "sentry-sample_rate=0.01337,sentry-user_id=Am%C3%A9lie,"
+            "sentry-sample_rate=0.01337,sentry-user_id=Am%C3%A9lie,sentry-foo=bar,"
             "other-vendor-value-1=foo;bar;baz,other-vendor-value-2=foo;bar;"
         ).split(",")
     )
